/********************************** (C) COPYRIGHT *******************************
 * File Name          : ch9431_example.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2023/10/10
 * Description        : This file contains ch9431 command example.
*********************************************************************************
* Copyright (c) 2023 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
#include "ch9431_example.h"

/* 每个文件单独debug打印的开关，置0可以禁止本文件内部打印 */
#define DEBUG_PRINT_IN_THIS_FILE 1
#if DEBUG_PRINT_IN_THIS_FILE
    #define PRINTF(...) printf(__VA_ARGS__)
#else
    #define PRINTF(...) do {} while (0)
#endif

/*********************************************************************
 * @fn      CH9431_Receive_Buffer1
 *
 * @brief   read data from can bus.
 *
 * @param   CAN_RX_Buf - the pointer to data buffer.
 * @param   rxbndlc - the pointer to rxbndlc_t.
 *
 * @return  0 - failed.
 *          1 - success.
 */
uint8_t CH9431_Receive_Buffer1(uint8_t *CAN_RX_Buf, rxbndlc_t *rxbndlc)
{
    uint8_t i = 0;
    sysint_t sysintf;

    sysintf.value = CH9431_ReadByte(R8_SYSINTF);

    if (sysintf.RX1I != 0)
    {
        rxbndlc->value = CH9431_ReadByte(R8_RXB1DLC);

        if ((rxbndlc->RTR) == 0) /* 判断非远程帧 */
        {
            while (i < rxbndlc->DLC)
            {
                CAN_RX_Buf[i] = CH9431_ReadByte(R8_RXB1D0 + i);        /* 把CAN接收到的数据放入指定缓冲区 */
                i++;
            }
        }

        return 1;
    }
    return 0;
}

/*********************************************************************
 * @fn      CH9431_Receive_Buffer0
 *
 * @brief   read data from can bus.
 *
 * @param   CAN_RX_Buf - the pointer to data buffer.
 * @param   rxbndlc - the pointer to rxbndlc_t.
 *
 * @return  0 - failed.
 *          1 - success.
 */
uint8_t CH9431_Receive_Buffer0(uint8_t *CAN_RX_Buf, rxbndlc_t *rxbndlc)
{
    uint8_t i = 0;
    sysint_t sysintf;

    sysintf.value = CH9431_ReadByte(R8_SYSINTF);

    if (sysintf.RX0I != 0)
    {
        rxbndlc->value = CH9431_ReadByte(R8_RXB0DLC);

        if ((rxbndlc->RTR) == 0) /* 判断非远程帧 */
        {
            while (i < rxbndlc->DLC)
            {
                CAN_RX_Buf[i] = CH9431_ReadByte(R8_RXB0D0 + i);     /* 把CAN接收到的数据放入指定缓冲区 */
                i++;
            }
        }

        return 1;
    }
    return 0;
}

/*********************************************************************
 * @fn      CH9431_Send_Buffer0
 *
 * @brief   send data to can bus.
 *
 * @param   CAN_TX_Buf - the pointer to data buffer.
 * @param   len - the data length.
 * @param   rtr - 0 or 1, 0 will send a data frame, 1 will send a remote transmit request.
 *
 * @return  0xff- failed.
 *          others- success.
 */
uint8_t CH9431_Send_Buffer0(uint8_t *CAN_TX_Buf, uint8_t len, uint8_t rtr)
{
    uint8_t j, dly, count;
    txbnctrl_t txbnctrl;

    count = 0;

    while (count < len)
    {
        dly = 0;
        while (1)
        {
            txbnctrl.value = CH9431_ReadByte(R8_TXB0CTRL);
            if (txbnctrl.TXREQ == 0)        /* 等待TXREQ标志清零 */
            {
                break;
            }
            _port_ch9431_delay_ms(1);
            dly++;
            if (dly > 50)
            {
                goto error;
            }
        }

        if (rtr == 0)
        {
            j = 0;

            while (j < 8)
            {
                CH9431_WriteByte(R8_TXB0D0 + j, CAN_TX_Buf[count++]);     /* 将待发送的数据写入发送缓冲寄存器 */
                j++;
                if (count >= len)
                {
                    break;
                }
            }
        }
        else
        {
            if (len > 8)
            {
                j = 8;
            }
            else
            {
                j = len;
            }
        }

        do
        {
            txbndlc_t txbndlc = {.value = 0};
            txbndlc.DLC = j;
            txbndlc.RTR = rtr;
            CH9431_WriteByte(R8_TXB0DLC, txbndlc.value);    /* 将本帧待发送的数据长度和远程帧请求位写入发送缓冲器0的发送长度寄存器 */
        }
        while (0);
#if USE_TXRTSPIN
        _port_ch9431_tx0rts_low();
        _port_ch9431_delay_ms(1);
        _port_ch9431_tx0rts_high();
#else
        do
        {
            txbnctrl_t txbnctrl_mask = {.value = 0};
            txbnctrl_mask.TXREQ = 1;
            txbnctrl.TXREQ = 1;
            CH9431_BIT_Modify(R8_TXB0CTRL, txbnctrl_mask.value, txbnctrl.value);
        }
        while (0);      /* 请求发送报文 */
#endif
        return 0;
    }

error:
    PRINTF("send 0 failed\n");
    return 0xff;
}

/*********************************************************************
 * @fn      CH9431_Send_Buffer1
 *
 * @brief   send data to can bus.
 *
 * @param   CAN_TX_Buf - the pointer to data buffer.
 * @param   len - the data length.
 * @param   rtr - 0 or 1, 0 will send a data frame, 1 will send a remote transmit request.
 *
 * @return  0xff- failed.
 *          others- success.
 */
uint8_t CH9431_Send_Buffer1(uint8_t *CAN_TX_Buf, uint8_t len, uint8_t rtr)
{
    uint8_t j, dly, count;
    txbnctrl_t txbnctrl;

    count = 0;

    while (count < len)
    {
        dly = 0;
        while (1)
        {
            txbnctrl.value = CH9431_ReadByte(R8_TXB1CTRL);
            if (txbnctrl.TXREQ == 0)        /* 等待TXREQ标志清零 */
            {
                break;
            }
            _port_ch9431_delay_ms(1);
            dly++;
            if (dly > 50)
            {
                goto error;
            }
        }

        if (rtr == 0)
        {
            j = 0;

            while (j < 8)
            {
                CH9431_WriteByte(R8_TXB1D0 + j, CAN_TX_Buf[count++]);     /* 将待发送的数据写入发送缓冲寄存器 */
                j++;
                if (count >= len)
                {
                    break;
                }
            }
        }
        else
        {
            if (len > 8)
            {
                j = 8;
            }
            else
            {
                j = len;
            }
        }

        do
        {
            txbndlc_t txbndlc = {.value = 0};
            txbndlc.DLC = j;
            txbndlc.RTR = rtr;
            CH9431_WriteByte(R8_TXB1DLC, txbndlc.value);    /* 将本帧待发送的数据长度和远程帧请求位写入发送缓冲器1的发送长度寄存器 */
        }
        while (0);
#if USE_TXRTSPIN
        _port_ch9431_tx1rts_low();
        _port_ch9431_delay_ms(1);
        _port_ch9431_tx1rts_high();
#else
        do
        {
            txbnctrl_t txbnctrl_mask = {.value = 0};
            txbnctrl_mask.TXREQ = 1;
            txbnctrl.TXREQ = 1;
            CH9431_BIT_Modify(R8_TXB1CTRL, txbnctrl_mask.value, txbnctrl.value);
        }
        while (0);      /* 请求发送报文 */
#endif
        return 0;
    }

error:
    PRINTF("send 1 failed\n");
    return 0xff;
}

/*********************************************************************
 * @fn      CH9431_Send_Buffer2
 *
 * @brief   send data to can bus.
 *
 * @param   CAN_TX_Buf - the pointer to data buffer.
 * @param   len - the data length.
 * @param   rtr - 0 or 1, 0 will send a data frame, 1 will send a remote transmit request.
 *
 * @return  0xff- failed.
 *          others- success.
 */
uint8_t CH9431_Send_Buffer2(uint8_t *CAN_TX_Buf, uint8_t len, uint8_t rtr)
{
    uint8_t j, dly, count;
    txbnctrl_t txbnctrl;

    count = 0;

    while (count < len)
    {
        dly = 0;
        while (1)
        {
            txbnctrl.value = CH9431_ReadByte(R8_TXB2CTRL);
            if (txbnctrl.TXREQ == 0)        /* 等待TXREQ标志清零 */
            {
                break;
            }
            _port_ch9431_delay_ms(1);
            dly++;
            if (dly > 50)
            {
                goto error;
            }
        }

        if (rtr == 0)
        {
            j = 0;

            while (j < 8)
            {
                CH9431_WriteByte(R8_TXB2D0 + j, CAN_TX_Buf[count++]);     /* 将待发送的数据写入发送缓冲寄存器 */
                _port_ch9431_delay_us(1);
                j++;
                if (count >= len)
                {
                    break;
                }
            }
        }
        else
        {
            if (len > 8)
            {
                j = 8;
            }
            else
            {
                j = len;
            }
        }

        do
        {
            txbndlc_t txbndlc = {.value = 0};
            txbndlc.DLC = j;
            txbndlc.RTR = rtr;
            CH9431_WriteByte(R8_TXB2DLC, txbndlc.value);    /* 将本帧待发送的数据长度和远程帧请求位写入发送缓冲器2的发送长度寄存器 */
            _port_ch9431_delay_us(1);
        }
        while (0);

#if USE_TXRTSPIN
        _port_ch9431_tx2rts_low();
        _port_ch9431_delay_ms(1);
        _port_ch9431_tx2rts_high();
#else
        do
        {
            txbnctrl_t txbnctrl_mask = {.value = 0};
            txbnctrl_mask.TXREQ = 1;
            txbnctrl.TXREQ = 1;
            CH9431_BIT_Modify(R8_TXB2CTRL, txbnctrl_mask.value, txbnctrl.value);
            _port_ch9431_delay_us(1);
        }
        while (0);      /* 请求发送报文 */
#endif
        return 0;
    }

error:
    PRINTF("send 2 failed\n");
    return 0xff;
}

/*********************************************************************
 * @fn      CH9431_Init
 *
 * @brief   init CH9431.
 *
 * @param   none.
 *
 * @return  none.
 */
void CH9431_Init(void)
{
    uint8_t set_count;

init:
    set_count = 0;

#if 0
    /* 硬件复位 */
    CH9431_HardReset();
#else
    /* 软件复位 */
    CH9431_Reset();              /* 发送复位指令软件复位MCP2515 */
#endif

    /* 设置波特率：20Mhz / ((btimer2.TS1+1+btimer3.TS2+1+1)*(btimer1.BRP+1)) */
    do
    {
        btimer1_t btimer1 = {.value = 0};
        btimer1.BRP = 3;
        CH9431_WriteByte(R8_BTIMER1, btimer1.value);
        _port_ch9431_delay_us(1);
        PRINTF("btimer1 set ok:%02x\n", btimer1.value);
    }
    while (0);

    do
    {
        btimer2_t btimer2 = {.value = 0};
        btimer2.SJW = 0;           /* SJW为0，长度为1TQ */
        btimer2.TS1 = 4;           /* PHSEG1为4，长度为5TQ */
        CH9431_WriteByte(R8_BTIMER2, btimer2.value);
        _port_ch9431_delay_us(1);
        PRINTF("btimer2 set ok:%02x\n", btimer2.value);
    }
    while (0);

    do
    {
        btimer3_t btimer3 = {.value = 0};
        btimer3.TS2 = 3;        /* PHSEG2为3，长度为4TQ */
        CH9431_WriteByte(R8_BTIMER3, btimer3.value);
        _port_ch9431_delay_us(1);
        PRINTF("btimer3 set ok:%02x\n", btimer3.value);
    }
    while (0);        /* 设置波特率结束：20Mhz / ((4+1+3+1+1)*(3+1)) = 500k */

    /* 设置报文发送ID 0 */
    do
    {
        txbn_t txbn = {.SIDH = 0, .SIDL = 0, .EIDH = 0, .EIDL = 0};
        txbn.SID = 0x123;       /* 标准帧ID */
        txbn.EID = 0x12345;     /* 扩展帧ID */
        txbn.EXIDE = 0;         /* 1:报文将发送扩展标识符,0:报文将发送标准标识符 */

        CH9431_WriteByte(R8_TXB0SIDL, txbn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB0SIDH, txbn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB0EIDL, txbn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB0EIDH, txbn.EIDH);
        _port_ch9431_delay_us(1);
        PRINTF("txb0 set ok\n");
    }
    while (0);

    /* 设置报文发送ID 1 */
    do
    {
        txbn_t txbn = {.SIDH = 0, .SIDL = 0, .EIDH = 0, .EIDL = 0};
        txbn.SID = 0x23;        /* 标准帧ID */
        txbn.EID = 0x2345;      /* 扩展帧ID */
        txbn.EXIDE = 0;         /* 1:报文将发送扩展标识符,0:报文将发送标准标识符 */

        CH9431_WriteByte(R8_TXB1SIDL, txbn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB1SIDH, txbn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB1EIDL, txbn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB1EIDH, txbn.EIDH);
        _port_ch9431_delay_us(1);
        PRINTF("txb1 set ok\n");
    }
    while (0);

    /* 设置报文发送ID 2 */
    do
    {
        txbn_t txbn = {.SIDH = 0, .SIDL = 0, .EIDH = 0, .EIDL = 0};
        txbn.SID = 0x3;         /* 标准帧ID */
        txbn.EID = 0x345;       /* 扩展帧ID */
        txbn.EXIDE = 1;         /* 1:报文将发送扩展标识符,0:报文将发送标准标识符 */

        CH9431_WriteByte(R8_TXB2SIDL, txbn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB2SIDH, txbn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB2EIDL, txbn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_TXB2EIDH, txbn.EIDH);
        _port_ch9431_delay_us(1);
        PRINTF("txb2 set ok\n");
    }
    while (0);

    /* 设置TXRTS，是否支持使用PIN控制发送 */
    do
    {
        txrtsctrl_t txrtsctrl = {.value = 0};
        txrtsctrl.FIT = 3;
        txrtsctrl.B0RTSM = 1;
        txrtsctrl.B1RTSM = 1;
        txrtsctrl.B2RTSM = 0;
        CH9431_BIT_Modify(R8_TXRTSCTRL, txrtsctrl.value, txrtsctrl.value);
        _port_ch9431_delay_us(1);
        PRINTF("txrtsctrl set ok\n");
    }
    while (0);

    do
    {
        /* 清空接收缓冲器0的标准标识符高低位、拓展标识符高低位 */
        rxbn_t rxbn = {.SIDH = 0, .SIDL = 0, .EIDH = 0, .EIDL = 0};
        CH9431_WriteByte(R8_RXB0SIDL, rxbn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXB0SIDH, rxbn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXB0EIDL, rxbn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXB0EIDH, rxbn.EIDH);
        _port_ch9431_delay_us(1);
        PRINTF("rxb0 set ok\n");
    }
    while (0);

    do
    {
        /* 仅接收拓展标识符的有效信息 */
        rxb0ctrl_t rxb0ctrl = {.value = 0};
        rxb0ctrl.BUKT = 1;          /* 1:如果RXB0满， RXB0接收到的报文将被滚存至RXB1, 0不滚存 */
        CH9431_WriteByte(R8_RXB0CTRL, rxb0ctrl.value);
        _port_ch9431_delay_us(1);
        PRINTF("rxb0ctrl set ok\n");
    }
    while (0);

    /* 设置接收滤波器 0 */
    do
    {
        rxfn_t rxfn;

        rxfn.SID = 0x123;
        rxfn.EID = 0x12345;
        rxfn.EN = 1;                /* 使能该滤波器 */
        rxfn.EXIDE = 0;             /*0 = 报文滤波仅应用于标准帧， 1 = 报文滤波仅应用于扩展帧*/

        CH9431_WriteByte(R8_RXF0SIDL, rxfn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF0SIDH, rxfn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF0EIDL, rxfn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF0EIDH, rxfn.EIDH);
        _port_ch9431_delay_us(1);

        PRINTF("rxf0 set ok\n");
    }
    while (0);

    /* 设置接收滤波器 1 */
    do
    {
        rxfn_t rxfn;

        rxfn.SID = 0x123;
        rxfn.EID = 0x12345;
        rxfn.EN = 1;                /* 使能该滤波器 */
        rxfn.EXIDE = 0;             /*0 = 报文滤波仅应用于标准帧， 1 = 报文滤波仅应用于扩展帧*/

        CH9431_WriteByte(R8_RXF1SIDL, rxfn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF1SIDH, rxfn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF1EIDL, rxfn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF1EIDH, rxfn.EIDH);
        _port_ch9431_delay_us(1);

        PRINTF("rxf1 set ok\n");
    }
    while (0);

    /* 设置接收屏蔽器 0 */
    do
    {
        rxmn_t rxmn;

        rxmn.SID = 0;
        rxmn.EID = 0;

        CH9431_WriteByte(R8_RXM0SIDL, rxmn.SIDL);       /* 配置验收屏蔽寄存器n标准标识符低位 */
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXM0SIDH, rxmn.SIDH);       /* 配置验收屏蔽寄存器n标准标识符高位 */
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXM0EIDL, rxmn.EIDL);       /* 配置验收滤波寄存器n拓展标识符低位 */
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXM0EIDH, rxmn.EIDH);       /* 配置验收滤波寄存器n拓展标识符高位 */
        _port_ch9431_delay_us(1);

        PRINTF("rxm0 set ok\n");
    }
    while (0);

    /* 设置接收滤波器 2 */
    do
    {
        rxfn_t rxfn;

        rxfn.SID = 0x123;
        rxfn.EID = 0x12345;
        rxfn.EN = 1;                /* 使能该滤波器 */
        rxfn.EXIDE = 0;             /*0 = 报文滤波仅应用于标准帧， 1 = 报文滤波仅应用于扩展帧*/

        CH9431_WriteByte(R8_RXF2SIDL, rxfn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF2SIDH, rxfn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF2EIDL, rxfn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF2EIDH, rxfn.EIDH);
        _port_ch9431_delay_us(1);

        PRINTF("rxf2 set ok\n");
    }
    while (0);

    /* 设置接收滤波器 3 */
    do
    {
        rxfn_t rxfn;

        rxfn.SID = 0x123;
        rxfn.EID = 0x12345;
        rxfn.EN = 1;                /* 使能该滤波器 */
        rxfn.EXIDE = 1;             /*0 = 报文滤波仅应用于标准帧， 1 = 报文滤波仅应用于扩展帧*/

        CH9431_WriteByte(R8_RXF3SIDL, rxfn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF3SIDH, rxfn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF3EIDL, rxfn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF3EIDH, rxfn.EIDH);
        _port_ch9431_delay_us(1);

        PRINTF("rxf3 set ok\n");
    }
    while (0);

    /* 设置接收滤波器 4 */
    do
    {
        rxfn_t rxfn;

        rxfn.SID = 0x123;
        rxfn.EID = 0x12345;
        rxfn.EN = 1;                /* 使能该滤波器 */
        rxfn.EXIDE = 1;             /*0 = 报文滤波仅应用于标准帧， 1 = 报文滤波仅应用于扩展帧*/

        CH9431_WriteByte(R8_RXF4SIDL, rxfn.SIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF4SIDH, rxfn.SIDH);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF4EIDL, rxfn.EIDL);
        _port_ch9431_delay_us(1);
        CH9431_WriteByte(R8_RXF4EIDH, rxfn.EIDH);
        _port_ch9431_delay_us(1);

        PRINTF("rxf4 set ok\n");
    }
    while (0);

    /* 设置接收滤波器 5 */
    do
    {
        rxfn_t rxfn;

        rxfn.SID = 0x123;
        rxfn.EID = 0x12345;
        rxfn.EN = 1;                /* 使能该滤波器 */
        rxfn.EXIDE = 1;             /*0 = 报文滤波仅应用于标准帧， 1 = 报文滤波仅应用于扩展帧*/

        CH9431_WriteByte(R8_RXF5SIDL, rxfn.SIDL);
        CH9431_WriteByte(R8_RXF5SIDH, rxfn.SIDH);
        CH9431_WriteByte(R8_RXF5EIDL, rxfn.EIDL);
        CH9431_WriteByte(R8_RXF5EIDH, rxfn.EIDH);

        PRINTF("rxf5 set ok\n");
    }
    while (0);

    /* 设置接收屏蔽器 1 */
    do
    {
        rxmn_t rxmn;

        rxmn.SID = 0;
        rxmn.EID = 0;

        CH9431_WriteByte(R8_RXM1SIDL, rxmn.SIDL);       /* 配置验收屏蔽寄存器n标准标识符低位 */
        CH9431_WriteByte(R8_RXM1SIDH, rxmn.SIDH);       /* 配置验收屏蔽寄存器n标准标识符高位 */
        CH9431_WriteByte(R8_RXM1EIDL, rxmn.EIDL);       /* 配置验收滤波寄存器n拓展标识符低位 */
        CH9431_WriteByte(R8_RXM1EIDH, rxmn.EIDH);       /* 配置验收滤波寄存器n拓展标识符高位 */

        PRINTF("rxm1 set ok\n");
    }
    while (0);

    /* 默认清空中断标志位 */
    do
    {
        sysint_t sysintf = {.value = 0};
        CH9431_WriteByte(R8_SYSINTF, sysintf.value);    /* 清空CAN中断标志寄存器的所有位(必须由MCU清空) */

        PRINTF("sysintf set ok\n");
    }
    while (0);

    /* 设置中断使能位 */
    do
    {
        sysint_t sysinte = {.value = 0};
        sysinte.RX0I = 1;                               /* 配置CAN中断使能寄存器的接收缓冲器0满中断使能,其它位禁止中断 */
        sysinte.RX1I = 1;                               /* 配置CAN中断使能寄存器的接收缓冲器1满中断使能,其它位禁止中断 */
        CH9431_WriteByte(R8_SYSINTE, sysinte.value);   /* 清空CAN中断标志寄存器的所有位(必须由MCU清空) */
        PRINTF("sysinte set ok\n");
    }
    while (0);

set_sysctrl:
    set_count++;

    /* 设置工作模式 */
    do
    {
        sysctrl_t sysctrl = {.value = 0};
        sysctrl.REQOP = SYSCTRL_REQOP_NORMAL;           /* 正常工作模式 */
        sysctrl.ABAT = 0;                               /* 请求中止所有报文发送 */
        sysctrl.OSM = 0;                                /* 1 报文仅尝试发送一次, 0 如有需要，报文会重新发送 */
        sysctrl.CLKPRE = 1;                             /* 2分频 */
        sysctrl.CLKEN = 1;                              /* 1 CLKOUT引脚使能, 0 CLKOUT引脚禁止 */
        CH9431_WriteByte(R8_SYSCTRL, sysctrl.value);
        PRINTF("sysctrl set ok\n");
    }
    while (0);

    _port_ch9431_delay_ms(2);

    do
    {
        sysstat_t sysstat;

        sysstat.value = CH9431_ReadByte(R8_SYSSTAT);

        if (sysstat.OPMOD != SYSCTRL_REQOP_NORMAL)
        {
            if (set_count < 200)
            {
                PRINTF("R8_SYSCTRL set error: %02x, retry\n", sysstat.value);
                goto set_sysctrl;
            }
            else
            {
                PRINTF("chip error, retry init\n");
                goto init;
            }
        }
        else
        {
            PRINTF("init ok\n");
        }
    }
    while (0);

}
